// Cross-Browser Rich Text Editor
// Version 3.14
// Written by Kevin Roth (http://www.kevinroth.com/rte/)
// License: http://creativecommons.org/licenses/by/2.5/

//init globals
var lang = "en";
var encoding = "UTF-8";
var debugMode = false;
var isRichText = false;
var defaultWidth = 500;
var defaultHeight = 200;
var allRTEs = "";
var currentRTE = "";
var selectionText = "";
var rng;
var lastCommand;
var maxLoops = 20;
var loopCnt = 0;
var baseUrl = window.parent.document.location.protocol + "//" + document.domain + "/";

var imagesPath = "./lib/rte2html/css/img/";
var includesPathCSS = "./lib/rte2html/css/";
var includesPathHTML = "./lib/rte2html/htm/";
var cssFile = "";
var generateXHTML = true;
var encodeHTML = false;

//browser vars
var ua = navigator.userAgent.toLowerCase();
var isIE = ((ua.indexOf("msie") != -1) && (ua.indexOf("opera") == -1) && (ua.indexOf("webtv") == -1));
var ieVersion = parseFloat(ua.substring(ua.indexOf('msie ') + 5));
//var isGecko = (ua.indexOf("gecko") != -1);
var isGecko = (navigator.product == 'Gecko');
var geckoRv = geckoGetRv();
var isSafari = (ua.indexOf("safari") != -1);
var isIphone = (ua.indexOf("iphone") != -1);
var isIpad = (ua.indexOf("ipad") != -1);
//var safariVersion = parseFloat(ua.substring(ua.lastIndexOf("safari/") + 7));
var webkitVersion = get_webkit_version();
if (webkitVersion) {
	webkitVersion = webkitVersion["major"];
} else {
	webkitVersion = null;
}
var isKonqueror = (ua.indexOf("konqueror") != -1);
var konquerorVersion = parseFloat(ua.substring(ua.indexOf('konqueror/') + 10));
var isOpera = (ua.indexOf("opera") != -1);
var isNetscape = (ua.indexOf("netscape") != -1);
var netscapeVersion = parseFloat(ua.substring(ua.lastIndexOf('/') + 1));

//command vars
var toolbar1Enabled = true;
var toolbar2Enabled = true;

var cmdFormatBlockEnabled = true;
var cmdFontNameEnabled = true;
var cmdFontSizeEnabled = true;
var cmdIncreaseFontSizeEnabled = false;
var cmdDecreaseFontSizeEnabled = false;

var cmdBoldEnabled = true;
var cmdItalicEnabled = true;
var cmdUnderlineEnabled = true;
var cmdStrikethroughEnabled = true;
var cmdSuperscriptEnabled = true;
var cmdSubscriptEnabled = true;

var cmdJustifyLeftEnabled = true;
var cmdJustifyCenterEnabled = true;
var cmdJustifyRightEnabled = true;
var cmdJustifyFullEnabled = true;

var cmdInsertHorizontalRuleEnabled = true;
var cmdInsertOrderedListEnabled = true;
var cmdInsertUnorderedListEnabled = true;

var cmdOutdentEnabled = true;
var cmdIndentEnabled = true;
var cmdForeColorEnabled = true;
var cmdHiliteColorEnabled = true;
var cmdInsertHTMLEnabled = true;
var cmdSpellCheckEnabled = false;
var cmdCreateLinkEnabled = true;
var cmdInsertImageEnabled = true;

var cmdCutEnabled = true;
var cmdCopyEnabled = true;
var cmdPasteEnabled = true;
var cmdUndoEnabled = true;
var cmdRedoEnabled = true;
var cmdRemoveFormatEnabled = true;
var cmdUnlinkEnabled = true;


function initRTE(container, imgPath, cssPath, htmlPath, css, genXHTML, encHTML) {
	//set browser vars
	var colorIFrame = null;
	try {
		//set global vars
		
		if (imgPath !=  null){ //else use default
		  imagesPath = imgPath;
		}
		
		if (cssPath != null){//else use default
		  includesPathCSS = cssPath;
		}
		
		if (htmlPath != null){//else use default
		  includesPathHTML = htmlPath;
		}
		
		cssFile = css;
		generateXHTML = genXHTML;
		if (encHTML) encodeHTML = true;
		
		//check to see if designMode mode is available and disable unsupported commands
		if (document.designMode) {
			if (document.getElementById && !isIE && !isSafari && !isKonqueror) {
				isRichText = true;
				if (isGecko || isOpera) {
					cmdIncreaseFontSizeEnabled = true;
					cmdDecreaseFontSizeEnabled = true;
					cmdCutEnabled = false;
					cmdCopyEnabled = false;
					cmdPasteEnabled = false;
				}
				if (isNetscape && netscapeVersion < 8) {
					cmdInsertHTMLEnabled = false;
				}
			} else if (isIE && ieVersion >= 5.5) {
				isRichText = true;
				cmdSpellCheckEnabled = true;
			} else if (isSafari && !isIphone && !isIpad) {
				if (webkitVersion >= 420) {
					isRichText = true;
				} else if (webkitVersion >= 312) {
					//Safari 1.3+ is capable of designMode, Safari 1.3 = webkit build 312
					isRichText = true;
					toolbar1Enabled = false;
					cmdStrikethroughEnabled = false;
					cmdJustifyFullEnabled = false;
					cmdInsertHorizontalRuleEnabled = false;
					cmdInsertOrderedListEnabled = false;
					cmdInsertUnorderedListEnabled = false;
					cmdOutdentEnabled = false;
					cmdIndentEnabled = false;
					cmdInsertImageEnabled = false;
					cmdPasteEnabled = false;
					cmdInsertHTMLEnabled = false;
					cmdCreateLinkEnabled = false;
					cmdUnlinkEnabled = false;
					cmdRemoveFormatEnabled = false;
				}
			} else if (isSafari && (isIphone || isIpad)){ //iOS case, this is code from Magnus!
				isRichText = true;
			}
		}
		
		if (isRichText) { 
		   colorIFrame = '<iframe width="150" height="100" id="cp" src="' + includesPathHTML + 'palette.htm" z-index="7000" marginwidth="0" marginheight="0" scrolling="no" frameborder="0" style="visibility:hidden; position: absolute;"></iframe>';
		  
			//document.writeln('<style type="text/css">@import "' + includesPathCSS + 'rte.css";</style>');
			//document.writeln('<iframe width="150" height="100" id="cp" src="' + includesPathHTML + 'palette.htm" marginwidth="0" marginheight="0" scrolling="no" frameborder="0" style="visibility:hidden; position: absolute;"></iframe>');
			if (isIE) {
				document.onmouseover = raiseButton;
				document.onmouseout  = normalButton;
				document.onmousedown = lowerButton;
				document.onmouseup   = raiseButton;
			}
		}
		
		//for testing standard textarea, uncomment the following line
		//isRichText = false;
	} catch (e) {
		if (debugMode) alert(e);
	}
	
	return colorIFrame;
}

function richTextEditor(id) {
	//creates a richTextEditor object
	this.rteID = id;
	
	//set defaults
	this.html = "";
	this.width = defaultWidth;
	this.height = defaultHeight;
	this.readOnly = false;
	this.toolbar1 = true;
	this.toolbar2 = true;
	
	this.cmdFormatBlock = true;
	this.cmdFontName = true;
	this.cmdFontSize = true;
	this.cmdIncreaseFontSize = false;
	this.cmdDecreaseFontSize = false;
	
	this.cmdBold = true;
	this.cmdItalic = true;
	this.cmdUnderline = true;
	this.cmdStrikethrough = false;
	this.cmdSuperscript = false;
	this.cmdSubscript = false;
	
	this.cmdJustifyLeft = true;
	this.cmdJustifyCenter = true;
	this.cmdJustifyRight = true;
	this.cmdJustifyFull = false;
	
	this.cmdInsertHorizontalRule = true;
	this.cmdInsertOrderedList = true;
	this.cmdInsertUnorderedList = true;
	
	this.cmdOutdent = true;
	this.cmdIndent = true;
	this.cmdForeColor = true;
	this.cmdHiliteColor = true;
	this.cmdInsertLink = true;
	this.cmdInsertImage = true;
	this.cmdInsertSpecialChars = true;
	this.cmdInsertTable = true;
	this.cmdSpellcheck = true;
	
	this.cmdCut = false;
	this.cmdCopy = false;
	this.cmdPaste = false;
	this.cmdUndo = false;
	this.cmdRedo = false;
	this.cmdRemoveFormat = false;
	this.cmdUnlink = false;
	
	this.toggleSrc = true;
	
	//add methods
	this.build = build;
}

function build() {
	with (this) {
		if (isRichText) {
			if (this.readOnly) {
				this.toolbar1 = false;
				this.toolbar2 = false;
				this.toggleSrc = false;
			}

			var editorHTML = writeRichText(this);
			enableDesignMode(this.rteID, htmlDecode(this.html), this.readOnly);
			
			if (isGecko && !this.readOnly) {
				//set focus on the RTE
				//needed for firefox < 3 or ul and ol buttons do not work
				currentRTE = this.rteID;
				if (geckoRv < 1.09) {
					insertHTML("<br>");
					rteCommand(this.rteID, "undo");
				}
			}
			
			//uncheck toggle source checkbox
			//TODO:mb Do we need this?
			//if (this.toggleSrc) document.getElementById("chkSrc" + this.rteID).checked = false;
			
			return editorHTML;
		} else {
			writePlainText(this);
		}
	}
}

function writePlainText(rte) {
	if (!rte.readOnly) {
		document.writeln('<textarea name="' + rte.rteID + '" id="' + rte.rteID + '" style="width: ' + rte.width + 'px; height: ' + rte.height + 'px;">' + rte.html + '</textarea>');
	} else {
		document.writeln('<textarea name="' + rte.rteID + '" id="' + rte.rteID + '" style="width: ' + rte.width + 'px; height: ' + rte.height + 'px;" readonly>' + rte.html + '</textarea>');
	}
}

function writeRichText(rte) {
	try {
		if (allRTEs.length > 0) allRTEs += ";";
		allRTEs += rte.rteID;
		
		var result = '<table width="' + rte.width + '" cellpadding="0" cellspacing="0" style="border: 1px solid #000;">'+
		'	<tr>'+
		'		<td>';
		if (toolbar1Enabled && rte.toolbar1) {
			result += '		<table class="rteBack" cellpadding="2" cellspacing="0" id="toolbar1_' + rte.rteID + '" width="100%">'+
			'			<tr>';
			if (cmdFormatBlockEnabled && rte.cmdFormatBlock) {
				result += '				<td>'+
				'					<select id="formatblock_' + rte.rteID + '" onchange="selectFont(\'' + rte.rteID + '\', this.id);">'+
				'						<option value="">[Style]</option>'+
				'						<option value="<p>">Paragraph &lt;p&gt;</option>'+
				'						<option value="<h1>">Heading 1 &lt;h1&gt;</option>'+
				'						<option value="<h2>">Heading 2 &lt;h2&gt;</option>'+
				'						<option value="<h3>">Heading 3 &lt;h3&gt;</option>'+
				'						<option value="<h4>">Heading 4 &lt;h4&gt;</option>'+
				'						<option value="<h5>">Heading 5 &lt;h5&gt;</option>'+
				'						<option value="<h6>">Heading 6 &lt;h6&gt;</option>'+
				'						<option value="<address>">Address &lt;ADDR&gt;</option>'+
				'						<option value="<pre>">Formatted &lt;pre&gt;</option>'+
				'					</select>'+
				'				</td>';
			}
			if (cmdFontNameEnabled && rte.cmdFontName) {
				result+= '				<td>'+
				'					<select id="fontname_' + rte.rteID + '" onchange="selectFont(\'' + rte.rteID + '\', this.id)">'+
				'						<option value="Font" selected>[Font]</option>'+
				'						<option value="Arial, Helvetica, sans-serif">Arial</option>'+
				'						<option value="Courier New, Courier, mono">Courier New</option>'+
				'						<option value="Times New Roman, Times, serif">Times New Roman</option>'+
				'						<option value="Verdana, Arial, Helvetica, sans-serif">Verdana</option>'+
				'					</select>'+
				'				</td>';
			}
			if (cmdFontSizeEnabled && rte.cmdFontSize) {
				result += '				<td>'+
				'					<select id="fontsize_' + rte.rteID + '" onchange="selectFont(\'' + rte.rteID + '\', this.id);">'+
				'						<option value="Size">[Size]</option>'+
				'						<option value="1">1</option>'+
				'						<option value="2">2</option>'+
				'						<option value="3">3</option>'+
				'						<option value="4">4</option>'+
				'						<option value="5">5</option>'+
				'						<option value="6">6</option>'+
				'						<option value="7">7</option>'+
				'					</select>'+
				'				</td>';
			}
			if (cmdIncreaseFontSizeEnabled && rte.cmdIncreaseFontSize)
				result+= '				<td><img class="rteImage" src="' + imagesPath + 'increase_font.gif" width="25" height="24" alt="Increase Font Size" title="Increase Font Size" onmousedown="rteCommand(\'' + rte.rteID + '\', \'increasefontsize\')"></td>';
			if (cmdDecreaseFontSizeEnabled && rte.cmdDecreaseFontSize)
				result+= '				<td><img class="rteImage" src="' + imagesPath + 'decrease_font.gif" width="25" height="24" alt="Decrease Font Size" title="Decrease Font Size" onmousedown="rteCommand(\'' + rte.rteID + '\', \'decreasefontsize\')"></td>'+
			'				<td width="100%"></td>'+
			'			</tr>'+
			'		</table>';
		}
		if (rte.toolbar2) {
			result+='		<table class="rteBack" cellpadding="0" cellspacing="0" id="toolbar2_' + rte.rteID + '" width="100%">'+
			'			<tr>';
			if (cmdBoldEnabled && rte.cmdBold)
				result+= '				<td><img class="rteImage" src="' + imagesPath + 'bold.gif" width="25" height="24" alt="Bold" title="Bold" onmousedown="rteCommand(\'' + rte.rteID + '\', \'bold\')"></td>';
			if (cmdItalicEnabled && rte.cmdItalic)
				result+= '				<td><img class="rteImage" src="' + imagesPath + 'italic.gif" width="25" height="24" alt="Italic" title="Italic" onmousedown="rteCommand(\'' + rte.rteID + '\', \'italic\')"></td>';
			if (cmdUnderlineEnabled && rte.cmdUnderline)
				result+= '				<td><img class="rteImage" src="' + imagesPath + 'underline.gif" width="25" height="24" alt="Underline" title="Underline" onmousedown="rteCommand(\'' + rte.rteID + '\', \'underline\')"></td>';
			if (cmdStrikethroughEnabled && rte.cmdStrikethrough)
				result+= '				<td><img class="rteImage" src="' + imagesPath + 'strikethrough.gif" width="25" height="24" alt="Strikethrough" title="Strikethrough" onmousedown="rteCommand(\'' + rte.rteID + '\', \'strikethrough\', \'\')"></td>';
			if (cmdSuperscriptEnabled && rte.cmdSuperscript)
				result+= '				<td><img class="rteImage" src="' + imagesPath + 'superscript.gif" width="25" height="24" alt="Superscript" title="Superscript" onmousedown="rteCommand(\'' + rte.rteID + '\', \'superscript\')"></td>';
			if (cmdSubscriptEnabled && rte.cmdSubscript)
				result+= '				<td><img class="rteImage" src="' + imagesPath + 'subscript.gif" width="25" height="24" alt="Subscript" title="Subscript" onmousedown="rteCommand(\'' + rte.rteID + '\', \'subscript\')"></td>';
//			document.writeln('				<td><img class="rteVertSep" src="' + imagesPath + 'blackdot.gif" width="1" height="20" border="0" alt=""></td>');
			if (cmdJustifyLeftEnabled && rte.cmdJustifyLeft)
				result+= '				<td><img class="rteImage" src="' + imagesPath + 'left_just.gif" width="25" height="24" alt="Align Left" title="Align Left" onmousedown="rteCommand(\'' + rte.rteID + '\', \'justifyleft\')"></td>';
			if (cmdJustifyCenterEnabled && rte.cmdJustifyCenter)
				result+= '				<td><img class="rteImage" src="' + imagesPath + 'centre.gif" width="25" height="24" alt="Center" title="Center" onmousedown="rteCommand(\'' + rte.rteID + '\', \'justifycenter\')"></td>';
			if (cmdJustifyRightEnabled && rte.cmdJustifyRight)
				result+= '				<td><img class="rteImage" src="' + imagesPath + 'right_just.gif" width="25" height="24" alt="Align Right" title="Align Right" onmousedown="rteCommand(\'' + rte.rteID + '\', \'justifyright\')"></td>';
			if (cmdJustifyFullEnabled && rte.cmdJustifyFull)
				result+= '				<td><img class="rteImage" src="' + imagesPath + 'justifyfull.gif" width="25" height="24" alt="Justify Full" title="Justify Full" onmousedown="rteCommand(\'' + rte.rteID + '\', \'justifyfull\')"></td>';
			if (cmdInsertHorizontalRuleEnabled && rte.cmdInsertHorizontalRule)
				result+= '				<td><img class="rteImage" src="' + imagesPath + 'hr.gif" width="25" height="24" alt="Horizontal Rule" title="Horizontal Rule" onmousedown="rteCommand(\'' + rte.rteID + '\', \'inserthorizontalrule\')"></td>';
			if (cmdInsertOrderedListEnabled && rte.cmdInsertOrderedList)
				result+= '				<td><img class="rteImage" src="' + imagesPath + 'numbered_list.gif" width="25" height="24" alt="Ordered List" title="Ordered List" onmousedown="rteCommand(\'' + rte.rteID + '\', \'insertorderedlist\')"></td>';
			if (cmdInsertUnorderedListEnabled && rte.cmdInsertUnorderedList)
				result+= '				<td><img class="rteImage" src="' + imagesPath + 'list.gif" width="25" height="24" alt="Unordered List" title="Unordered List" onmousedown="rteCommand(\'' + rte.rteID + '\', \'insertunorderedlist\')"></td>';
			if (cmdOutdentEnabled && rte.cmdOutdent)
				result+= '				<td><img class="rteImage" src="' + imagesPath + 'outdent.gif" width="25" height="24" alt="Outdent" title="Outdent" onmousedown="rteCommand(\'' + rte.rteID + '\', \'outdent\')"></td>';
			if (cmdIndentEnabled && rte.cmdIndent)
				result+= '				<td><img class="rteImage" src="' + imagesPath + 'indent.gif" width="25" height="24" alt="Indent" title="Indent" onmousedown="rteCommand(\'' + rte.rteID + '\', \'indent\')"></td>';
			if (cmdForeColorEnabled && rte.cmdForeColor)
				result+= '				<td><div id="forecolor_' + rte.rteID + '"><img class="rteImage" src="' + imagesPath + 'textcolor.gif" width="25" height="24" alt="Text Color" title="Text Color" onmousedown="dlgColorPalette(\'' + rte.rteID + '\', \'forecolor\', \'\'); return false;"></div></td>';
			if (cmdHiliteColorEnabled && rte.cmdHiliteColor)
				result+= '				<td><div id="hilitecolor_' + rte.rteID + '"><img class="rteImage" src="' + imagesPath + 'bgcolor.gif" width="25" height="24" alt="Background Color" title="Background Color" onmousedown="dlgColorPalette(\'' + rte.rteID + '\', \'hilitecolor\', \'\'); return false;"></div></td>';
			if ((cmdInsertHTMLEnabled || cmdCreateLinkEnabled) && rte.cmdInsertLink)
				result+= '				<td><img class="rteImage" src="' + imagesPath + 'hyperlink.gif" width="25" height="24" alt="Insert Link" title="Insert Link" onmousedown="dlgInsertLink(\'' + rte.rteID + '\')"></td>';
			if (cmdUnlinkEnabled && rte.cmdUnlink)
				result+= '				<td><img class="rteImage" src="' + imagesPath + 'unlink.gif" width="25" height="24" alt="Remove Link" title="Remove Link" onmousedown="rteCommand(\'' + rte.rteID + '\', \'unlink\')"></td>';
			if (cmdInsertImageEnabled && rte.cmdInsertImage)
				result+= '				<td><img class="rteImage" src="' + imagesPath + 'image.gif" width="25" height="24" alt="Add Image" title="Add Image" onmousedown="addImage(\'' + rte.rteID + '\')"></td>';
			if (cmdInsertHTMLEnabled && rte.cmdInsertSpecialChars)
				result+= '				<td><img class="rteImage" src="' + imagesPath + 'special_chars.gif" width="25" height="24" alt="Insert Special Character" title="Insert Special Character" onmousedown="dlgInsertSpecialChar(\'' + rte.rteID + '\')"></td>';
			if (cmdInsertHTMLEnabled && rte.cmdInsertTable && !isOpera)
				result+= '				<td><div id="table_' + rte.rteID + '"><img class="rteImage" src="' + imagesPath + 'insert_table.gif" width="25" height="24" alt="Insert Table" title="Insert Table" onmousedown="dlgInsertTable(\'' + rte.rteID + '\')"></div></td>';
			if (cmdSpellCheckEnabled && rte.cmdSpellcheck)
				result+= '				<td><img class="rteImage" src="' + imagesPath + 'spellcheck.gif" width="25" height="24" alt="Spell Check" title="Spell Check" onmousedown="checkspell()"></td>';
			if (cmdCutEnabled && rte.cmdCut)
				result+= '				<td><img class="rteImage" src="' + imagesPath + 'cut.gif" width="25" height="24" alt="Cut" title="Cut" onmousedown="rteCommand(\'' + rte.rteID + '\', \'cut\')"></td>';
			if (cmdCopyEnabled && rte.cmdCopy)
				result+= '				<td><img class="rteImage" src="' + imagesPath + 'copy.gif" width="25" height="24" alt="Copy" title="Copy" onmousedown="rteCommand(\'' + rte.rteID + '\', \'copy\')"></td>';
			if (cmdPasteEnabled && rte.cmdPaste)
				result+= '				<td><img class="rteImage" src="' + imagesPath + 'paste.gif" width="25" height="24" alt="Paste" title="Paste" onmousedown="rteCommand(\'' + rte.rteID + '\', \'paste\')"></td>';
			if (cmdUndoEnabled && rte.cmdUndo)
				result+= '				<td><img class="rteImage" src="' + imagesPath + 'undo.gif" width="25" height="24" alt="Undo" title="Undo" onmousedown="rteCommand(\'' + rte.rteID + '\', \'undo\')"></td>';
			if (cmdRedoEnabled && rte.cmdRedo)
				result+= '				<td><img class="rteImage" src="' + imagesPath + 'redo.gif" width="25" height="24" alt="Redo" title="Redo" onmousedown="rteCommand(\'' + rte.rteID + '\', \'redo\')"></td>';
			if (cmdRemoveFormatEnabled && rte.cmdRemoveFormat)
				result+= '				<td><img class="rteImage" src="' + imagesPath + 'removeformat.gif" width="25" height="24" alt="Remove Formatting" title="Remove Formatting" onmousedown="rteCommand(\'' + rte.rteID + '\', \'removeformat\')"></td>';
			result+= '				<td width="100%"></td>'+
			'			</tr>'+
			'		</table>';
		}
		result+= '<iframe id="' + rte.rteID + '" name="' + rte.rteID + '" src="' + includesPathHTML + 'blank.htm" frameborder="0" style="width: 100%; height: ' + rte.height + 'px; margin: 0; padding: 0;"></iframe>'+
		'		</td>'+
		'	</tr>'+
		'</table>'+
		'<div style="margin: 0; padding: 0;">';
		if (!rte.readOnly && rte.toggleSrc){ 
			result += '<input type="checkbox" id="chkSrc' + rte.rteID + '" onclick="toggleHTMLSrc(\'' + rte.rteID + '\',' + rte.toolbar1 + ',' + rte.toolbar2 + ');" />&nbsp;<label for="chkSrc' + rte.rteID + '">View Source</label>'+
		'<input type="hidden" id="hdn' + rte.rteID + '" name="' + rte.rteID + '" value="">'+
		'</div>';
		}
	} catch (e) {
		if (debugMode) alert(e);
	}
	
	return result;
}

function enableDesignMode(rte, html, readOnly) {
	try {
		if (cssFile.length > 0) {
			var frameStyle = "<style type=\"text/css\">@import url(" + cssFile + ");</style>\n";
		} else {
			var frameStyle = "<style type=\"text/css\">\n";
			frameStyle += "body {\n";
			frameStyle += "	background: #FFF;\n";
			frameStyle += "}\n";
			frameStyle += "</style>\n";
		}
		
		var frameHtml = "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\n";
		frameHtml += "<html id=\"" + rte + "\" xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\">\n";
		frameHtml += "<head>\n";
		frameHtml += "<meta http-equiv=\"content-type\" content=\"text/html; charset=UTF-8\" />\n";
		frameHtml += "<base href=\"" + baseUrl + "\" />\n";
		//if (!isGecko) frameHtml += frameStyle;
		frameHtml += frameStyle;
		frameHtml += "</head>\n";
		frameHtml += "<body style=\"margin: 0; padding: 0;\">\n";
		frameHtml += html + "\n";
		//if (isGecko) frameHtml += frameStyle;
		frameHtml += "</body>\n";
		frameHtml += "</html>";
		
		var oRTE = document.getElementById(rte);
		
		try {
			if (isGecko) {
				if (!readOnly) oRTE.contentDocument.designMode = "on";
				try {
					var oRTEDoc = oRTE.contentWindow.document;
					oRTEDoc.open("text/html","replace");
					oRTEDoc.write(frameHtml);
					oRTEDoc.close();
					if (!readOnly) oRTEDoc.addEventListener("keypress", geckoKeyPress, true);
				} catch (e) {
					alert("Error preloading content.");
				}
				loopCnt = 0;
			} else if (oRTE.contentWindow) {
				//IE 5.5+
				try {
					var oRTEDoc = oRTE.contentWindow.document;
					oRTEDoc.open("text/html","replace");
					oRTEDoc.write(frameHtml);
					oRTEDoc.close();
					if (!readOnly && isIE) oRTEDoc.attachEvent("onkeypress", evt_ie_keypress);
				} catch (e) {
					alert("Error preloading content.");
				}
				if (!readOnly) oRTEDoc.designMode = "on";
				loopCnt = 0;
			} else {
				//IE5 and Opera
				var oRTEDoc = oRTE.document;
				oRTEDoc.open("text/html","replace");
				oRTEDoc.write(frameHtml);
				oRTEDoc.close();
				if (!readOnly && isIE) oRTEDoc.attachEvent("onkeypress", evt_ie_keypress);
				if (!readOnly) oRTEDoc.designMode = "on";
				loopCnt = 0;
			}
			
			/*if (iMOD.isTouch()){
			  oRTE.contentWindow.addEventListener('touchstart', handleLinks, false);
			}
			else{
			  oRTE.contentWindow.addEventListener('mouseup', handleLinks, true);
			}*/	
		} catch (e) {
		  if (debugMode) alert(e);
		  
			//some browsers may take some time to enable design mode.
			//Keep looping until able to set.
			if (loopCnt < maxLoops) {
			  //old code from kevinroth
				//setTimeout("enableDesignMode('" + rte + "', '" + html + "', " + readOnly + ");", 100);
				
				//new code from magnus
				setTimeout(function() {
          enableDesignMode(rte, html, readOnly);
        }, 100);
				
				loopCnt += 1;
			} else {
				alert("Error enabling designMode.");
			}
		}
	} catch (e) {
      if (debugMode) alert(e);
	}
}

/*function handleLinks(e){
     //remove old info link if necessary!
     $("#show_info_link_id").remove();
  
     var iframe = document.getElementById("second_iframe");
     var iframeBody = (iframe.contentDocument || iframe.contentWindow.document).body;
     var caretPos = getCaretCharacterOffsetWithin(iframeBody);
     iMOD.debug("caretPos = " + caretPos);
     
     var theBody = iframeBody.innerHTML;
     theBody = theBody.replace(/<br>/g, ' '); //important: method stripHTML would remove those breaks!

     var strippedBody = stripHTML(htmlDecode(theBody));
     strippedBody = strippedBody.replace(/&nbsp;/g, ' '); //important: &nbsp; must be replaced with an empty string with length 1!
     
     //iMOD.debug("substr = " + strippedBody.substring(0, caretPos));
     
     var extractedString = jQuery.trim(extractString(strippedBody, caretPos));
     iMOD.debug("extractedString = " + extractedString);
     var isBrowserString = checkURL(extractedString); 
     iMOD.debug("isBrowserString = " + isBrowserString);
     var isEmail = isValidEmailAddress(extractedString);
     iMOD.debug("isEmail = " + isEmail);
     
     if (isBrowserString || isEmail) {
      if (isBrowserString) {
      //link MUST start with http://
        if (extractedString.indexOf("http://") != 0 
          && extractedString.indexOf("https://") != 0
          && extractedString.indexOf("ftp://") != 0) {
         extractedString = "http://" + extractedString;
        }
      }
      
      // add a div tag with link
      var theID = "show_info_link_id";
      var theDiv = document.createElement('div');
      theDiv.id = theID;
      $(theDiv).addClass('info_link');
      theDiv.style.zIndex = "10003";
      theDiv.style.position = "absolute";
      
      var theLink = document.createElement('a');
      if (isBrowserString) {
        theLink.setAttribute('href', extractedString);
        theLink.setAttribute('target', "_blank");
        $(theLink).html(extractedString);
      } else{
        var theEmailCmd = "mailto:"+extractedString;
        theLink.setAttribute('href', theEmailCmd); 
        $(theLink).html(theEmailCmd);
      }
      
      $(theLink).addClass('info_link_href');
      
      theDiv.appendChild(theLink);
      document.body.appendChild(theDiv);
    
      $('a[href]', theDiv).click(function(evt) {
        iMOD.Ext.openUrl(event.target.href);
        $('#show_info_link_id').remove();
        return false;
      });
      
       var leftPos = e.clientX || 50;
      var topPos = e.clientY || 100;
     
      $(theDiv).css({left: leftPos, top: topPos}).show();
    }  
}*/

/*function getCaretCharacterOffsetWithin(element) {
    var doc = element.ownerDocument || element.document;
    var win = doc.defaultView || doc.parentWindow;
    var sel, range, preCaretRange, caretOffset = 0;
    if (typeof win.getSelection != "undefined") {
        sel = win.getSelection();
        if (sel.rangeCount) {
            range = sel.getRangeAt(0);
            preCaretRange = range.cloneRange();
            preCaretRange.selectNodeContents(element);
            preCaretRange.setEnd(range.endContainer, range.endOffset);
            caretOffset = preCaretRange.toString().length;
        }
    } else if ( (sel = doc.selection) && sel.type != "Control") {
        range = doc.selection.createRange();
        preCaretRange = doc.body.createTextRange();
        preCaretRange.moveToElementText(element);
        preCaretRange.setEndPoint("EndToEnd", textRange);
        caretOffset = preCaretTextRange.text.length;
    }
    return caretOffset;
}*/

//TODO:mb This is copied from link_textarea.js
/*function extractString(wellU, selStart) {
  var firstBlank = selStart;
  var lastBlank = selStart;
  var i = selStart;
   
  // if you click on last possible index or AFTER it, no link should occur
  if (selStart == wellU.length) {
    return ' ';
  }
   
  // find firstBlank 
  while (i > 0) {
    i--;
    //alert(i);
    var nextChar = wellU.substring(i, i+1);   
    //alert(nextChar);
    if (nextChar == ' ' || nextChar == '\n' || nextChar == '\r'){
      //alert("Bingo");
      firstBlank = i;
      break;
    }
    
    if (i==0){ //first word
      firstBlank = 0;
    }
  }
   
   //find lastBlank
  i = selStart-1;
  while (i < wellU.length) {
    i++
    //alert(i);
    var nextChar = wellU.substring(i, i+1);
    //alert(nextChar);
    if (nextChar == ' ' || nextChar == '\n' || nextChar == '\r') {
      //alert("Bingo 2");
      lastBlank = i;
      break;
    }
    
    if (i == wellU.length){ //last word
      lastBlank = wellU.length;
    }
  }
   
  var result = wellU.substring(firstBlank, lastBlank);
  return result;
}*/

//TODO:mb This is copied from link_textarea.js
/*function checkURL(value) {
  var urlregex = new RegExp("^(http:\/\/|https:\/\/|ftp:\/\/|www.){1}([0-9A-Za-z]+\.)");
  return urlregex.test(value)
}*/
   
//TODO:mb This is copied from link_textarea.js   
/*function isValidEmailAddress(str) {
  var at="@";
  var dot=".";
  var lat=str.indexOf(at);
  var lstr=str.length;
  var ldot=str.indexOf(dot);
  if (str.indexOf(at)==-1) {
    return false;
  }

  if (str.indexOf(at)==-1 || str.indexOf(at)==0 || str.indexOf(at)==lstr) {
    return false;
  }

  if (str.indexOf(dot)==-1 || str.indexOf(dot)==0 || str.indexOf(dot)==lstr) {
    return false;
  }

  if (str.indexOf(at,(lat+1))!=-1) {
    return false;
  }

  if (str.substring(lat-1,lat)==dot || str.substring(lat+1,lat+2)==dot) {
    return false;
  }

  if (str.indexOf(dot,(lat+2))==-1) {
    return false;
  }
    
  if (str.indexOf(" ")!=-1){
    return false;
  }

  return true;         
}*/

//TODO:mb No idea for iOS in connection with designMode!?
/*function getCoordsXY(ev) {
  var coords = new Object();	
  if (ev.clientX && ev.clientY){
  	alert("1");
  	coords.xcoord = ev.clientX;
  	coords.ycoord = ev.clientY;
  } else if(ev.pageX) {
  	alert("2");
    coords.xcoord = ev.pageX;
    coords.ycoord = ev.pageY;
  } else if(window.event && window.event.clientX) {
  	alert("3");
    var isStrictMode = document.compatMode && document.compatMode != 'BackCompat' ? true : false;
    var scrollX = isStrictMode ? document.documentElement.scrollLeft : document.body.scrollLeft;
    var scrollY = isStrictMode ? document.documentElement.scrollTop : document.body.scrollTop;
    coords.xcoord = window.event.clientX + scrollX;
    coords.ycoord = window.event.clientY + scrollY;
  } else if (window.event && window.event.targetTouches.length > 0){
  	alert("4");
  	var touchEvt = window.event.targetTouches[0];
  	coords.xcoord = touchEvt.pageX;
  	coords.ycoord = touchEvt.pageY;
  }
  
  alert(coords.ycoord);
  return coords;
}*/

function updateRTE(rte) {
	try {
		if (isRichText) {
			//if viewing source, switch back to design view
			if (document.getElementById("chkSrc" + rte) && document.getElementById("chkSrc" + rte).checked) document.getElementById("chkSrc" + rte).click();
			setHiddenVal(rte);
		} else {
			return;
		}
	} catch (e) {
		if (debugMode) alert(e);
	}
}

function updateRTEs() {
	try {
		if (allRTEs != "") {
			var vRTEs = allRTEs.split(";");
			for (var i = 0; i < vRTEs.length; i++) {
				updateRTE(vRTEs[i]);
			}
		}
	} catch (e) {
		if (debugMode) alert(e);
	}
}

function rteCommand(rte, command, option) {
	//function to perform command
	try {
		var oRTE = document.getElementById(rte);
		if (oRTE.contentWindow) {
			oRTE.contentWindow.focus();
			oRTE.contentWindow.document.execCommand(command, false, option);
		} else {
			oRTE.document.focus();
		  	oRTE.document.execCommand(command, false, option);
		}
		loopCnt = 0;
		
		return false;
	} catch (e) {
		if (debugMode) alert(e);
		//Keep looping until able to set.
		if (loopCnt < maxLoops) {
			setTimeout("rteCommand('" + rte + "', '" + command + "', '" + option + "');", 100);
			loopCnt += 1;
		} else {
			alert("Error executing command.");
		}
	}
}

function dlgColorPalette(rte, command) {
	//function to display or hide color palettes
	try {
		setRange(rte);
		
		//get dialog position
		var buttonElement = document.getElementById(command + '_' + rte);
		//var iLeftPos = findPosX(buttonElement);
		var mainDiv = document.getElementById('role_welcome_popup');
		var leftShift = 300; //TODO:mb Hard coded, parametrize it? Maybe other value on iOS?
		var iLeftPos = parseInt(String(mainDiv.style.left).substring(0, mainDiv.style.left.length-2)) + leftShift;
		
		var iTopPos = findPosY(buttonElement) + buttonElement.offsetHeight;
		var oDialog = document.getElementById('cp');
		
		oDialog.style.left = iLeftPos + "px";
		oDialog.style.top = iTopPos + "px";
		
		if ((command == lastCommand) && (rte == currentRTE)) {
			//if current command dialog is currently open, close it
			if (oDialog.style.visibility == "hidden") {
				showHideElement('cp', 'show');
			} else {
				showHideElement('cp', 'hide');
			}
		} else {
			showHideElement('cp', 'show');
		}
		
		//save current values
		lastCommand = command;
		currentRTE = rte;
	} catch (e) {
		if (debugMode) alert(e);
	}
}

function dlgInsertTable(rte) {
	//function to open insert table dialog
	try {
		//save current values
		setRange(rte);
		currentRTE = rte;
		InsertTable = popUpWin(includesPathHTML + 'insert_table.htm', 'InsertTable', 360, 180, '');
		if (InsertTable) {
			setTimeout("InsertTable.focus()",100);
		} else {
			alert("Error: While Launching New Window...\nYour browser maybe blocking up Popup windows.\n\nPlease check your Popup Blocker Settings");
		}
	} catch (e) {
		if (debugMode) alert(e);
	}
}

function dlgInsertLink(rte) {
	//function to open insert link dialog
	try {
		if (cmdInsertHTMLEnabled) {
			//save current values
			setRange(rte);
			currentRTE = rte;
			InsertLink = popUpWin(includesPathHTML + 'insert_link.htm?selectionText=' + selectionText, 'InsertLink', 360, 180, '');
			if (InsertLink) {
				setTimeout("InsertLink.focus()",100);
			} else {
				alert("Error: While Launching New Window...\nYour browser maybe blocking up Popup windows.\n\nPlease check your Popup Blocker Settings");
			}
		} else {
			var url = prompt("Enter URL", "http://");
			rteCommand(rte, "createlink", url);
		}
	} catch (e) {
		if (debugMode) alert(e);
	}
}

function dlgInsertSpecialChar(rte) {
	//function to open insert special character dialog
	try {
		//save current values
		setRange(rte);
		currentRTE = rte;
		InsertSpecialChar = popUpWin(includesPathHTML + 'insert_special_char.htm', 'InsertSpecialChar', 360, 250, '');
		if (InsertSpecialChar) {
			setTimeout("InsertSpecialChar.focus()",100);
		} else {
			alert("Error: While Launching New Window...\nYour browser maybe blocking up Popup windows.\n\nPlease check your Popup Blocker Settings");
		}
	} catch (e) {
		if (debugMode) alert(e);
	}
}

function popUpWin (url, win, width, height, options) {
	try {
		var leftPos = (screen.availWidth - width) / 2;
		var topPos = (screen.availHeight - height) / 2;
		options += 'width=' + width + ',height=' + height + ',top=' + topPos + ',left=' + leftPos;
		return window.open(url, win, options);
	} catch (e) {
		if (debugMode) alert(e);
	}
}

function setColor(color) {
	//function to set color
	try {
		var rte = currentRTE;
		if (isSafari || isIE) {
			if (lastCommand == "hilitecolor") lastCommand = "backcolor";
			
			//retrieve selected range
			if (!isSafari) rng.select();
		}
		
		rteCommand(rte, lastCommand, color);
		showHideElement('cp', "hide");
	} catch (e) {
		if (debugMode) alert(e);
	}
}

function addImage(rte) {
	//function to add image
	try {
		imagePath = prompt('Enter Image URL:', 'http://');
		if ((imagePath != null) && (imagePath != "")) {
			rteCommand(rte, 'InsertImage', imagePath);
		}
	} catch (e) {
		if (debugMode) alert(e);
	}
}

//positioning functions courtesy of Peter-Paul Koch - http://www.quirksmode.org/
function findPosX(obj) {  
	var curleft = 0;
	if (obj.offsetParent) {
		curleft = obj.offsetLeft;
		while (obj = obj.offsetParent) {
			curleft += obj.offsetLeft;
		}
	}
	return curleft;
}

function findPosY(obj) {
	var curtop = 0;
	if (obj.offsetParent) {
		curtop = obj.offsetTop;
		while (obj = obj.offsetParent) {
			curtop += obj.offsetTop;
		}
	}
	return curtop;
}

function selectFont(rte, selectname) {
	//function to handle font changes
	try {
		var idx = document.getElementById(selectname).selectedIndex;
		// First one is always a label
		if (idx != 0) {
			var selected = document.getElementById(selectname).options[idx].value;
			var cmd = selectname.replace('_' + rte, '');
			rteCommand(rte, cmd, selected);
			document.getElementById(selectname).selectedIndex = 0;
		}
	} catch (e) {
		if (debugMode) alert(e);
	}
}

function insertHTML(html) {
	//function to add HTML -- thanks dannyuk1982
	try {
		if (document.all && !isOpera) {
			rng.pasteHTML(html);
			rng.collapse(false);
			rng.select();
		} else {
			rteCommand(currentRTE, 'inserthtml', html);
		}
	} catch (e) {
		if (debugMode) alert(e);
	}
}

function showHideElement(element, showHide) {
	//function to show or hide elements
	try {
		//element variable can be string or object
		if (document.getElementById(element)) {
			element = document.getElementById(element);
			if (showHide == "show") {
				if (element.id.substring(0,7) == "toolbar") element.style.display = "block";
				element.style.visibility = "visible";
			} else if (showHide == "hide") {
				if (element.id.substring(0,7) == "toolbar") element.style.display = "none";
				element.style.visibility = "hidden";
			}
		}
	} catch (e) {
		if (debugMode) alert(e);
	}
}

function setRange(rte) {
	//function to store range of current selection
	try {
		var oRTE = document.getElementById(rte);
		if (oRTE.contentWindow) {
			var oRTEDoc = oRTE.contentWindow.document;
			oRTE.contentWindow.focus();
		} else {
			var oRTEDoc = oRTE.document;
			oRTEDoc.focus();
		}
		
		if (document.all) {
			sel = oRTEDoc.selection;
			rng = sel.createRange();
			if (encodeHTML) {
				selectionText = htmlEncode(rng.text.toString());
			} else {
				selectionText = rng.text.toString();
			}
		} else if (document.getSelection) {
			rng = oRTEDoc.createRange();
			if (encodeHTML) {
				selectionText = htmlEncode(oRTEDoc.getSelection());
			} else {
				selectionText = oRTEDoc.getSelection();
			}
		}
	} catch (e) {
		if (debugMode) alert(e);
	}
}

function stripHTML(oldString) {
	//function to strip all html
	try {
		var newString = oldString.replace(/(<([^>]+)>)/ig,"");
		
		//replace carriage returns and line feeds
	   newString = newString.replace(/\r\n/g," ");
	   newString = newString.replace(/\n/g," ");
	   newString = newString.replace(/\r/g," ");
		
		//trim string
		newString = trim(newString);
		
		return newString;
	} catch (e) {
		if (debugMode) alert(e);
	}
}

function trim(sInString) {
	//removes leading and trailing spaces from the passed string
	try {
		sInString = sInString.replace(/^\s+/g, "");
		return sInString.replace(/\s+$/g, "");
	} catch (e) {
		if (debugMode) alert(e);
	}
}

function cleanWordContent(wordContent) {
	//strip comments
	wordContent = String(wordContent).replace(/<!--(.|\n)*?-->/g,"");
	
	//strip word tags
	wordContent = String(wordContent).replace(/<\\?\?xml[^>]*>/g,"");
	wordContent = String(wordContent).replace(/<\/?(.*?):[^>]*>/g,"");

	//strip other dangerous tags
	wordContent = String(wordContent).replace(/<\/?xml[^>]*>/g,"");
	wordContent = String(wordContent).replace(/<\/?meta[^>]*>/g,"");
	wordContent = String(wordContent).replace(/<\/?link[^>]*>/g,"");
	wordContent = String(wordContent).replace(/<\/?style[^>]*>/g,"");
//	wordContent = String(wordContent).replace(/&nbsp;/g,"");//<p>&nbsp;</p>
//	wordContent = String(wordContent).replace(/<\/?SPAN[^>]*>/g,"");
//	wordContent = String(wordContent).replace(/<\/?FONT[^>]*>/g,"");
//	wordContent = String(wordContent).replace(/<\/?STRONG[^>]*>/g,"");
//	wordContent = String(wordContent).replace(/<\/?P[^>]*><\/P>/g,"");
//	wordContent = String(wordContent).replace(/<\/?H1[^>]*>/g,"");
//	wordContent = String(wordContent).replace(/<\/?H2[^>]*>/g,"");
//	wordContent = String(wordContent).replace(/<\/?H3[^>]*>/g,"");
//	wordContent = String(wordContent).replace(/<\/?H4[^>]*>/g,"");
//	wordContent = String(wordContent).replace(/<\/?H5[^>]*>/g,"");
//	wordContent = String(wordContent).replace(/<\/?H6[^>]*>/g,"");
	
	//replace line breaks
	wordContent = String(wordContent).replace(/\n/g," ");

	return trim(wordContent);
}

function toggleHTMLSrc(rte, toolbar1, toolbar2) {
	//contributed by Bob Hutzel (thanks Bob!)
	try {
		var oHdnField = document.getElementById('hdn' + rte);
		var oRTE = document.getElementById(rte);
		if (oRTE.contentWindow) {
			var oRTEDoc = oRTE.contentWindow.document;
		} else {
			var oRTEDoc = oRTE.document;
		}
		
		if (document.getElementById("chkSrc" + rte).checked) {
			//we are checking the box, show source
			if (toolbar1) showHideElement("toolbar1_" + rte, "hide");
			if (toolbar2) showHideElement("toolbar2_" + rte, "hide");
			setHiddenVal(rte);
			if (encodeHTML) {
				var htmlSrc = oRTEDoc.createTextNode(htmlDecode(oHdnField.value));
			} else {
				var htmlSrc = oRTEDoc.createTextNode(oHdnField.value);
			}
			oRTEDoc.body.focus();
			oRTEDoc.body.innerHTML = "";
			oRTEDoc.body.appendChild(htmlSrc);
		} else {
			//we are unchecking the box, show rich text
			if (toolbar1) showHideElement("toolbar1_" + rte, "show");
			if (toolbar2) showHideElement("toolbar2_" + rte, "show");
			
			if (isIE) {
				var htmlSrc = oRTEDoc.body.innerText;
			} else {
				var htmlSrc = oRTEDoc.body.ownerDocument.createRange();
				htmlSrc.selectNodeContents(oRTEDoc.body);
				htmlSrc = htmlSrc.toString();
			}
			setHtmlSrc(rte, htmlSrc);
		}
	} catch (e) {
		if (debugMode) alert(e);
	}
}

function setHiddenVal(rte) {
	//set hidden form field value for current rte
	try {
		var oHdnField = document.getElementById('hdn' + rte);
		if (oHdnField.value == null) oHdnField.value = "";
		
		var html = getHtmlSrc(rte);
		html = cleanWordContent(html);
		
		if (generateXHTML) {
			//convert html output to xhtml (thanks Jacob Lee!)
			html = getXHTML(html);
		}
		
		//if there is no content (other than formatting) set value to nothing
		if (stripHTML(html.replace("&nbsp;", " ")) == "" &&
			html.toLowerCase().search("<hr") == -1 &&
			html.toLowerCase().search("<img") == -1) html = "";
		
		if (encodeHTML) {
			oHdnField.value = htmlEncode(html);
		} else {
			oHdnField.value = html;
		}
	} catch (e) {
		if (debugMode) alert(e);
	}
}

function getHtmlSrc(rte) {
	try {
		var oRTE = document.getElementById(rte);
		if (oRTE.contentWindow) {
			var oRTEDoc = oRTE.contentWindow.document;
		} else {
			var oRTEDoc = oRTE.document;
		}
		return oRTEDoc.body.innerHTML;
	} catch (e) {
		if (debugMode) alert(e);
	}
}

function setHtmlSrc(rte, html) {
	try {
		var oRTE = document.getElementById(rte);
		if (oRTE.contentWindow) {
			var oRTEDoc = oRTE.contentWindow.document;
		} else {
			var oRTEDoc = oRTE.document;
		}
		if (isIE) { //fix for IE
			var output = htmlEncode(html);
			output = output.replace("%3CP%3E%0D%0A%3CHR%3E", "%3CHR%3E");
			output = output.replace("%3CHR%3E%0D%0A%3C/P%3E", "%3CHR%3E");
			oRTEDoc.body.innerHTML = htmlDecode(output);
		} else {
			//replace char 160 with space
			html = strip160(html);
			oRTEDoc.body.innerHTML = html;
		}
	} catch (e) {
		if (debugMode) alert(e);
	}
}

function strip160(strHTML) {
	newString = '';
	for (i = 0; i < strHTML.length; i++) {
		if (strHTML.charCodeAt(i) == 160) {
			newString = newString + ' ';
		} else {
			newString = newString + strHTML.charAt(i);
		}
	}
	return newString;
}

function htmlEncode(text) {
	return text.replace(/\&/ig,  '&amp;').replace(/\</ig,  '&lt;').replace(/\>/ig,  '&gt;').replace(/\"/ig,  '&quot;');
}

function htmlDecode(text) {
	return text.replace(/\&amp\;/ig,  '&').replace(/\&lt\;/ig,  '<').replace(/\&gt\;/ig,  '>').replace(/\&quot\;/ig,  '"');
}

// webkit.org webkit detection scripts
function parse_webkit_version(version) {
	try {
		var bits = version.split(".");
		var is_nightly = (version[version.length - 1] == "+");
		if (is_nightly) {
			var minor = "+";
		} else {
			var minor = parseInt(bits[1]);
			// If minor is Not a Number (NaN) return an empty string
			if (isNaN(minor)) {
				minor = "";
			}
		}
		return {major: parseInt(bits[0]), minor: minor, is_nightly: is_nightly};
	} catch (e) {
		if (debugMode) alert(e);
	}
}

function get_webkit_version() {
	try {
		var regex = new RegExp("\\(.*\\) AppleWebKit/(.*) \\((.*)");
		var matches = regex.exec(navigator.userAgent);
		if (matches) {
			var webkit_version = parse_webkit_version(matches[1]);    
			return {major: webkit_version['major'], minor: webkit_version['minor'], is_nightly: webkit_version['is_nightly']};
		} else {
			return false;
		}
	} catch (e) {
		//if (debugMode) alert(e);
	}
}

//********************
//Gecko-Only Functions
//********************
function geckoKeyPress(evt) {
	//function to add bold, italic, and underline shortcut commands to gecko RTEs
	//contributed by Anti Veeranna (thanks Anti!)
	try {
		var rte = evt.target.id;
		
		if (evt.ctrlKey) {
			var key = String.fromCharCode(evt.charCode).toLowerCase();
			var cmd = '';
			switch (key) {
				case 'b': cmd = "bold"; break;
				case 'i': cmd = "italic"; break;
				case 'u': cmd = "underline"; break;
			};
	
			if (cmd) {
				rteCommand(rte, cmd);
				
				// stop the event bubble
				evt.preventDefault();
				evt.stopPropagation();
			}
	 	}
	} catch (e) {
		if (debugMode) alert(e);
	}
}

function geckoGetRv() {
	if (navigator.product != 'Gecko') {
		return -1;
	}
	var rvValue = 0;
	var ua      = navigator.userAgent.toLowerCase();
	var rvStart = ua.indexOf('rv:');
	var rvEnd   = ua.indexOf(')', rvStart);
	var rv      = ua.substring(rvStart+3, rvEnd);
	var rvParts = rv.split('.');
	var exp     = 1;
	
	for (var i = 0; i < rvParts.length; i++) {
		var val = parseInt(rvParts[i]);
		rvValue += val / exp;
		exp *= 100;
	}
	
	return rvValue;
}


//*****************
//IE-Only Functions
//*****************
function evt_ie_keypress(event) {
	try {
		ieKeyPress(event, rte);
	} catch (e) {
		if (debugMode) alert(e);
	}
} 

function ieKeyPress(evt, rte) {
	try {
		var key = (evt.which || evt.charCode || evt.keyCode);
		var stringKey = String.fromCharCode(key).toLowerCase();
		
		//the following breaks list and indentation functionality in IE (don't use)
		//	switch (key) {
		//		case 13:
		//			//insert <br> tag instead of <p>
		//			//change the key pressed to null
		//			evt.keyCode = 0;
		//			
		//			//insert <br> tag
		//			currentRTE = rte;
		//			insertHTML('<br>');
		//			break;
		//	};
	} catch (e) {
		if (debugMode) alert(e);
	}
}

function checkspell() {
	//function to perform spell check
	try {
		var tmpis = new ActiveXObject("ieSpell.ieSpellExtension");
		tmpis.CheckAllLinkedDocuments(document);
	} catch(e) {
		if(e.number==-2146827859) {
			if (confirm("ieSpell not detected.  Click Ok to go to download page."))
				window.open("http://www.iespell.com/download.php","DownLoad");
		} else {
			alert("Error Loading ieSpell: Exception " + e.number);
		}
	}
}

function raiseButton(e) {
	try {
		var el = window.event.srcElement;
		
		className = el.className;
		if (className == 'rteImage' || className == 'rteImageLowered') {
			el.className = 'rteImageRaised';
		}
	} catch (e) {
		if (debugMode) alert(e);
	}
}

function normalButton(e) {
	try {
		var el = window.event.srcElement;
		
		className = el.className;
		if (className == 'rteImageRaised' || className == 'rteImageLowered') {
			el.className = 'rteImage';
		}
	} catch (e) {
		if (debugMode) alert(e);
	}
}

function lowerButton(e) {
	try {
		var el = window.event.srcElement;
		
		className = el.className;
		if (className == 'rteImage' || className == 'rteImageRaised') {
			el.className = 'rteImageLowered';
		}
	} catch (e) {
		if (debugMode) alert(e);
	}
}
